import { ChangeDetectionStrategy, ChangeDetectorRef, Component, inject, OnDestroy, OnInit } from '@angular/core';
import { FaIconComponent } from '@fortawesome/angular-fontawesome';
import { PermissionDirective } from '../../../permissions/permission.directive';
import { TranslocoDirective, TranslocoPipe, TranslocoService } from '@jsverse/transloco';
import { RouterLink } from '@angular/router';
import {
    CardBodyComponent,
    CardComponent,
    CardFooterComponent,
    CardHeaderComponent,
    CardTitleDirective,
    ColComponent,
    ContainerComponent,
    DropdownDividerDirective,
    FormCheckComponent,
    FormCheckInputDirective,
    FormCheckLabelDirective,
    FormControlDirective,
    FormDirective,
    InputGroupComponent,
    InputGroupTextDirective,
    ModalService,
    NavComponent,
    NavItemComponent,
    RowComponent,
    TableDirective
} from '@coreui/angular';
import { XsButtonDirective } from '../../../layouts/coreui/xsbutton-directive/xsbutton.directive';
import { IndexPage } from '../../../pages.interface';
import {
    AgentconnectorPushRoot,
    AgentsPushParams,
    getDefaultAgentsPushParams,
    PushAgent
} from '../agentconnector.interface';
import { Subscription } from 'rxjs';
import { PermissionsService } from '../../../permissions/permissions.service';
import { AgentconnectorService } from '../agentconnector.service';
import { MatSort, MatSortHeader, Sort } from '@angular/material/sort';
import { PaginatorChangeEvent } from '../../../layouts/coreui/paginator/paginator.interface';
import { DebounceDirective } from '../../../directives/debounce.directive';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { AsyncPipe, NgForOf, NgIf } from '@angular/common';
import { TableLoaderComponent } from '../../../layouts/primeng/loading/table-loader/table-loader.component';
import { ItemSelectComponent } from '../../../layouts/coreui/select-all/item-select/item-select.component';
import { CopyToClipboardComponent } from '../../../layouts/coreui/copy-to-clipboard/copy-to-clipboard.component';
import { NoRecordsComponent } from '../../../layouts/coreui/no-records/no-records.component';
import {
    PaginateOrScrollComponent
} from '../../../layouts/coreui/paginator/paginate-or-scroll/paginate-or-scroll.component';
import { SelectAllComponent } from '../../../layouts/coreui/select-all/select-all.component';
import { DeleteAllItem } from '../../../layouts/coreui/delete-all-modal/delete-all.interface';
import { SelectionServiceService } from '../../../layouts/coreui/select-all/selection-service.service';
import { NotyService } from '../../../layouts/coreui/noty.service';
import { DeleteAllModalComponent } from '../../../layouts/coreui/delete-all-modal/delete-all-modal.component';
import { DELETE_SERVICE_TOKEN } from '../../../tokens/delete-injection.token';
import { ActionsButtonComponent } from '../../../components/actions-button/actions-button.component';
import {
    ActionsButtonElementComponent
} from '../../../components/actions-button-element/actions-button-element.component';

@Component({
    selector: 'oitc-agentconnector-push',
    imports: [
        FaIconComponent,
        PermissionDirective,
        TranslocoDirective,
        RouterLink,
        CardBodyComponent,
        CardComponent,
        CardHeaderComponent,
        CardTitleDirective,
        NavComponent,
        NavItemComponent,
        XsButtonDirective,
        CardFooterComponent,
        ColComponent,
        ContainerComponent,
        DebounceDirective,
        FormCheckComponent,
        FormCheckInputDirective,
        FormCheckLabelDirective,
        FormControlDirective,
        FormDirective,
        FormsModule,
        InputGroupComponent,
        InputGroupTextDirective,
        ReactiveFormsModule,
        RowComponent,
        TranslocoPipe,
        MatSort,
        NgIf,
        TableDirective,
        TableLoaderComponent,
        MatSortHeader,
        AsyncPipe,
        ItemSelectComponent,
        NgForOf,
        CopyToClipboardComponent,
        NoRecordsComponent,
        PaginateOrScrollComponent,
        SelectAllComponent,
        DeleteAllModalComponent,
        ActionsButtonComponent,
        ActionsButtonElementComponent,
        DropdownDividerDirective
    ],
    templateUrl: './agentconnector-push.component.html',
    styleUrl: './agentconnector-push.component.css',
    providers: [
        {provide: DELETE_SERVICE_TOKEN, useClass: AgentconnectorService} // Inject the UsersService into the DeleteAllModalComponent
    ],
    changeDetection: ChangeDetectionStrategy.OnPush
})
export class AgentconnectorPushComponent implements OnInit, OnDestroy, IndexPage {

    public agents?: AgentconnectorPushRoot;

    public host_assignment: boolean = false;
    public no_host_assignment: boolean = false;

    public hideFilter: boolean = true;
    public params: AgentsPushParams = getDefaultAgentsPushParams();

    public selectedItems: any[] = [];

    public readonly PermissionsService = inject(PermissionsService);

    public readonly AgentconnectorService = inject(AgentconnectorService);
    private readonly SelectionServiceService = inject(SelectionServiceService);
    private readonly TranslocoService: TranslocoService = inject(TranslocoService);
    private readonly notyService: NotyService = inject(NotyService);
    private readonly modalService = inject(ModalService);
    private cdr = inject(ChangeDetectorRef);
    private subscriptions: Subscription = new Subscription();

    public ngOnInit(): void {
        this.loadPushAgents();
    }

    public ngOnDestroy(): void {
        this.subscriptions.unsubscribe();
    }

    public loadPushAgents() {
        let hasHostAssignment: string | boolean = '';
        if (this.host_assignment !== this.no_host_assignment) {
            hasHostAssignment = this.host_assignment === true;
        }
        this.params['filter[hasHostAssignment]'] = hasHostAssignment;

        this.subscriptions.add(this.AgentconnectorService.getPushAgents(this.params)
            .subscribe((result) => {
                this.agents = result;
                this.cdr.markForCheck();
            })
        );
    }

    // Show or hide the filter
    // Called by (click) - no manual change detection required
    public toggleFilter() {
        this.hideFilter = !this.hideFilter;
    }

    // Callback when a filter has changed
    public onFilterChange(event: any) {
        this.params.page = 1;
        this.loadPushAgents();
    }

    // Callback when sort has changed
    public onSortChange(sort: Sort) {
        if (sort.direction) {
            this.params.sort = sort.active;
            this.params.direction = sort.direction;
            this.loadPushAgents();
        }
    }

    // Callback for Paginator or Scroll Index Component
    public onPaginatorChange(change: PaginatorChangeEvent): void {
        this.params.page = change.page;
        this.params.scroll = change.scroll;
        this.loadPushAgents();
    }

    public resetFilter() {
        this.params = getDefaultAgentsPushParams();
        this.loadPushAgents();
    }


    // Generic callback whenever a mass action (like delete all) has been finished
    public onMassActionComplete(success: boolean) {
        if (success) {
            this.loadPushAgents();
        }
    }

    // Open the Delete All Modal
    public toggleDeleteAllModal(agent?: PushAgent) {
        let items: DeleteAllItem[] = [];

        if (agent) {
            // User just want to delete a single command
            let name = agent.uuid;
            if (agent.HostMapping.display_name !== null) {
                name = agent.HostMapping.display_name;
            }

            items = [{
                id: Number(agent.id),
                displayName: String(name)
            }];
        } else {
            // User clicked on delete selected button
            items = this.SelectionServiceService.getSelectedItems().map((item): DeleteAllItem => {
                let name = item.uuid;
                if (item.HostMapping.display_name !== null) {
                    name = item.HostMapping.display_name;
                }

                return {
                    id: item.id,
                    displayName: String(name)
                };
            });
        }


        if (items.length === 0) {
            const message = this.TranslocoService.translate('No items selected!');
            this.notyService.genericError(message);
            return;
        }

        // Pass selection to the modal
        this.selectedItems = items;
        // open modal
        this.modalService.toggle({
            show: true,
            id: 'deleteAllModal',
        });
    }

}
