import { inject, Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { PROXY_PATH } from '../../tokens/proxy-path.token';
import { map, Observable } from 'rxjs';
import {
    DowntimeObject,
    DowntimesHostParams,
    DowntimesHostRoot,
    DowntimesServiceParams,
    DowntimesServiceRoot
} from './downtimes.interface';
import { CancelAllItem } from './cancel-hostdowntime-modal/cancel-hostdowntime.interface';

@Injectable({
    providedIn: 'root'
})
export class DowntimesService {


    private readonly http = inject(HttpClient);
    private readonly proxyPath = inject(PROXY_PATH);

    constructor() {
    }

    public getDowntimeTooltipDetails(objectUuid: string, type: 'hosts' | 'services'): Observable<DowntimeObject> {
        const proxyPath = this.proxyPath;

        const params: any = {
            angular: true,
            disableGlobalLoader: true
        };

        if (type === 'hosts') {
            params['uuid'] = objectUuid;
        } else {
            params['serviceuuid'] = objectUuid;
        }

        return this.http.get<{
            downtime: DowntimeObject
        }>(`${proxyPath}/${type}/browser.json`, {
            params: params
        }).pipe(
            map(data => {
                return data.downtime
            })
        )
    }

    // Generic function for the Delete All Modal
    public deleteHostdowntime(item: CancelAllItem, includeServices: boolean): Observable<Object> {
        const proxyPath = this.proxyPath;
        return this.http.post(`${proxyPath}/externalcommands/cancel_downtime.json?angular=true`, {
            internal_downtime_id: item.id,
            include_services: includeServices,
            type: 'host'
        });
    }

    // Generic function for the Delete All Modal
    public deleteServicedowntime(item: CancelAllItem, includeServices: boolean): Observable<Object> {
        const proxyPath = this.proxyPath;
        return this.http.post(`${proxyPath}/externalcommands/cancel_downtime.json?angular=true`, {
            internal_downtime_id: item.id,
            include_services: false,
            type: 'service'
        });
    }

    /**********************
     *    Host action    *
     **********************/
    public getDowntimesHost(params: DowntimesHostParams): Observable<DowntimesHostRoot> {
        const proxyPath = this.proxyPath;
        return this.http.get<DowntimesHostRoot>(`${proxyPath}/downtimes/host.json`, {
            params: params as {}
        }).pipe(
            map(data => {
                return data;
            })
        )
    }

    /**********************
     *   Service action   *
     **********************/
    public getDowntimesService(params: DowntimesServiceParams): Observable<DowntimesServiceRoot> {
        const proxyPath = this.proxyPath;
        return this.http.get<DowntimesServiceRoot>(`${proxyPath}/downtimes/service.json`, {
            params: params as {}
        }).pipe(
            map(data => {
                return data;
            })
        )
    }
}
