import { PaginateOrScroll } from '../../layouts/coreui/paginator/paginator.interface';

/**********************
 *    Index action    *
 **********************/
export interface HostchecksIndexParams {
    angular: true,
    scroll: boolean,
    sort: string,
    page: number,
    direction: 'asc' | 'desc' | '', // asc or desc
    'filter[Hostchecks.hostname]': string, // uuid
    'filter[Hostchecks.output]': string,
    'filter[Hostchecks.state][]': string[],
    'filter[Hostchecks.is_hardstate]': string | number,
    'filter[from]': Date | string,
    'filter[to]': Date | string,
}

export function getDefaultHostchecksIndexParams(): HostchecksIndexParams {
    let now = new Date();
    return {
        angular: true,
        scroll: true,
        sort: 'Hostchecks.start_time',
        page: 1,
        direction: 'desc',
        'filter[Hostchecks.hostname]': '',
        'filter[Hostchecks.output]': '',
        'filter[Hostchecks.state][]': [],
        'filter[Hostchecks.is_hardstate]': '',
        'filter[from]': new Date(now.getTime() - (3600 * 24 * 1000 * 30)),
        'filter[to]': new Date(now.getTime() + (3600 * 24 * 5)),
    }
}

export interface HostchecksIndexRoot extends PaginateOrScroll {
    all_hostchecks: Hostcheck[]
}

export interface Hostcheck {
    hostname: string
    start_time: string
    start_time_usec: number
    state: number
    is_hardstate: boolean
    end_time: string
    output: string
    timeout: number
    early_timeout: boolean
    latency: number
    execution_time: number
    perfdata: string
    command: string
    current_check_attempt: number
    max_check_attempts: number
    long_output: string
}


export interface HostStateFilter {
    recovery: boolean
    down: boolean
    unreachable: boolean
}

/**
 * Generic function that can be used by more actionms than notifications
 * @param state
 */
export function getHostStateForApi(state: HostStateFilter): string[] {
    let result = [];
    if (state.recovery) {
        result.push('recovery');
    }
    if (state.down) {
        result.push('down');
    }
    if (state.unreachable) {
        result.push('unreachable');
    }

    return result;
}
