import { PaginateOrScroll } from '../../layouts/coreui/paginator/paginator.interface';
import { HostMapping, Hoststatus, TimeperiodMapping } from '../hosts/hosts.interface';
import { ServiceBrowserAcknowledgement } from '../acknowledgements/acknowledgement.interface';
import { ServiceBrowserDowntime } from '../downtimes/downtimes.interface';

/**********************
 *    Index action    *
 **********************/
export interface ServicesIndexRoot extends PaginateOrScroll {
    servicestatus: ServicestatusIndex[]
    username: string
}

export interface Servicestatus {
    hostname: string
    service_description: string
    status_update_time: string
    output: string
    long_output: string
    perfdata: string
    current_state: number
    current_check_attempt: number
    max_check_attempts: number
    last_check: string
    next_check: string
    is_passive_check: boolean
    last_state_change: string
    last_hard_state_change: string
    last_hard_state: number
    is_hardstate: boolean
    last_notification: number
    next_notification: number
    notifications_enabled: boolean
    problem_has_been_acknowledged: boolean
    acknowledgement_type: number
    passive_checks_enabled: boolean
    active_checks_enabled: boolean
    event_handler_enabled: boolean
    flap_detection_enabled: boolean
    is_flapping: boolean
    latency: number
    execution_time: number
    scheduled_downtime_depth: number
    process_performance_data: boolean
    obsess_over_service: boolean
    normal_check_interval: number
    retry_check_interval: number
    check_timeperiod: string
    node_name: string
    last_time_ok: string
    last_time_warning: string
    last_time_critical: string
    last_time_unknown: string
    current_notification_number: number
    percent_state_change: number
    event_handler?: string
    check_command: string
    host_mapping: HostMapping
    service_mapping: ServiceMapping
    last_check_in_words: string
    next_check_in_words: string
    last_state_change_for_humans: string
    last_hard_state_change_for_humans: string
    normal_check_interval_for_humans: string
    retry_check_interval_for_humans: string
    output_html: string
    long_output_html: string
}

export interface ServicestatusIndex extends Servicestatus {
    hoststatus: Hoststatus
}

export interface ServicestatusBrowser extends Servicestatus {
    timeperiod_mapping: TimeperiodMapping
}

export interface ServiceMapping {
    id: number
    uuid: string
    display_name: string
    type_id: number
}


export interface ServicesBrowserRoot {
    servicestatus: ServicestatusBrowser
    acknowledgement: ServiceBrowserAcknowledgement | null
    downtime: ServiceBrowserDowntime | null
    Perfdata: {
        [key: string]: ServiceBrowserPerfdata
    }
    username: string
    _csrfToken: string
}

export interface ServiceBrowserPerfdata {
    current: string
    unit: string
    warning: string
    critical: string
    min: string
    max: any
    metric: string
}

export interface ServicesIndexParams {
    angular: true,
    scroll: boolean,
    sort: string,
    page: number,
    direction: 'asc' | 'desc' | ''
    'filter[Servicestatus.hostname]': string // uuuid

    'filter[Servicestatus.current_state][]': string[], // ok, warning, critical, unknown
    'filter[HostMapping.display_name]': string
    'filter[ServiceMapping.display_name]': string
    'filter[Servicestatus.output]': string
    'filter[Servicestatus.problem_has_been_acknowledged]': string | boolean,
    'filter[Servicestatus.scheduled_downtime_depth]': string | boolean,
    'filter[Servicestatus.is_hardstate]': string | boolean
}

export function getDefaultServicesIndexParams(): ServicesIndexParams {
    return {
        angular: true,
        scroll: true,
        sort: 'Servicestatus.current_state',
        page: 1,
        direction: 'desc',
        'filter[Servicestatus.hostname]': '',
        'filter[Servicestatus.current_state][]': [],
        'filter[HostMapping.display_name]': '',
        'filter[ServiceMapping.display_name]': '',
        'filter[Servicestatus.output]': '',
        'filter[Servicestatus.problem_has_been_acknowledged]': '',
        'filter[Servicestatus.scheduled_downtime_depth]': '',
        'filter[Servicestatus.is_hardstate]': ''
    }
}

export interface ServicesCurrentStateFilter {
    ok: boolean,
    warning: boolean,
    critical: boolean,
    unknown: boolean
}

export function getServiceCurrentStateForApi(currentState: ServicesCurrentStateFilter): string[] {
    let result = [];
    if (currentState.ok) {
        result.push('ok');
    }
    if (currentState.warning) {
        result.push('warning');
    }
    if (currentState.critical) {
        result.push('critical');
    }
    if (currentState.unknown) {
        result.push('unknown');
    }

    return result;
}


/***********************
 *    Automap action    *
 ************************/


export interface ServicesAutomapParams {
    'filter[Servicestatus.current_state][]': string[], // ok, warning, critical, unknown
    'filter[HostMapping.display_name]': string
    'filter[ServiceMapping.display_name]': string
}

export function getDefaultServicesAutomapParams(): ServicesAutomapParams {
    return {
        'filter[Servicestatus.current_state][]': [],
        'filter[HostMapping.display_name]': '',
        'filter[ServiceMapping.display_name]': '',
    }
}

export interface ServicesAutomapRoot {
    services: {
        hoststatus: AutomapHoststatus
        display_name: string
        hostname: string // host uuid
        servicestatus: AutomapServicesstatus[]
    }[]
    _csrfToken: string
}

export interface AutomapHoststatus {
    current_state: number
    is_flapping: 0 | 1
    is_hardstate: 0 | 1
    problem_has_been_acknowledged: 0 | 1
    scheduled_downtime_depth: number
    flap_detection_enabled: 0 | 1
}

export interface AutomapServicesstatus {
    hostname: string
    service_description: string
    current_state: number
    active_checks_enabled: boolean
    last_state_change: number
    last_check: number
    next_check: number
    output: string
    is_flapping: boolean
    is_hardstate: boolean
    problem_has_been_acknowledged: boolean
    scheduled_downtime_depth: number
    HostMapping: {
        display_name: string
    }
    ServiceMapping: {
        display_name: string
    }
    Hoststatus: AutomapHoststatus
}
