import { inject, Injectable } from '@angular/core';
import { catchError, map, Observable, of } from 'rxjs';
import { SelectKeyValue } from '../../layouts/primeng/select.interface';
import { HttpClient } from '@angular/common/http';
import { PROXY_PATH } from '../../tokens/proxy-path.token';
import {
    Acos,
    UsergroupDefault,
    UsergroupsEditGet,
    UsergroupsIndexParams,
    UsergroupsIndexRoot
} from './usergroups.interface';
import { DeleteAllItem } from '../../layouts/coreui/delete-all-modal/delete-all.interface';
import { GenericIdResponse, GenericResponseWrapper, GenericValidationError } from '../../generic-responses';

@Injectable({
    providedIn: 'root'
})
export class UsergroupsService {

    constructor() {
    }

    private readonly http: HttpClient = inject(HttpClient);
    private readonly proxyPath: string = inject(PROXY_PATH);

    public getIndex(params: UsergroupsIndexParams): Observable<UsergroupsIndexRoot> {
        return this.http.get<UsergroupsIndexRoot>(`${this.proxyPath}/usergroups/index.json?angular=true`, {
            params: params as {}
        }).pipe(
            map((data: UsergroupsIndexRoot) => {
                return data;
            })
        )
    }

    public loadAcos(): Observable<Acos> {
        return this.http.get<Acos>(`${this.proxyPath}/usergroups/add.json?angular=true`);
    }


    public loadUsergroups(): Observable<SelectKeyValue[]> {
        const proxyPath = this.proxyPath;
        return this.http.get<{ usergroups: SelectKeyValue[] }>(`${proxyPath}/users/loadUsergroups.json`, {
            params: {
                angular: true
            }
        }).pipe(
            map(data => {
                return data.usergroups;
            })
        )
    }

    public addUsergroup(usergroup: UsergroupDefault): Observable<GenericResponseWrapper> {
        return this.http.post<any>(`${this.proxyPath}/usergroups/add.json?angular=true`, usergroup)
            .pipe(
                map(data => {
                    return {
                        success: true,
                        data: data as GenericIdResponse
                    };
                }),
                catchError((error: any) => {
                    const err = error.error.error as GenericValidationError;
                    return of({
                        success: false,
                        data: err
                    });
                })
            );
    }

    public getEdit(id: number): Observable<UsergroupsEditGet> {
        return this.http.get<UsergroupsEditGet>(`${this.proxyPath}/usergroups/edit/${id}.json?angular=true`);
    }

    public updateUsergroup(post: UsergroupDefault): Observable<any> {
        return this.http.post<any>(`${this.proxyPath}/usergroups/edit/${post.Usergroup.id}.json?angular=true`, post)
            .pipe(
                map(data => {
                    return {
                        success: true,
                        data: data.usergroup as GenericIdResponse
                    };
                }),
                catchError((error: any) => {
                    const err = error.error.error as GenericValidationError;
                    return of({
                        success: false,
                        data: err
                    });
                })
            );
    }

    public delete(item: DeleteAllItem): Observable<Object> {
        return this.http.post(`${this.proxyPath}/usergroups/delete/${item.id}.json?angular=true`, {});
    }

}
