import { SelectKeyValue, SelectKeyValueString } from '../../layouts/primeng/select.interface';
import { PaginateOrScroll } from '../../layouts/coreui/paginator/paginator.interface';

export interface UsersIndexRoot extends PaginateOrScroll {
    users: User[]
    myUserId: number
    _csrfToken: string
}

export interface User {
    id: number
    username: string
    usergroup_id: number
    firstname: string
    lastname: string
    paginatorlength: number
    showstatsinmenu: boolean
    image: any
    dateformat: string
    timezone: string
    created: string
    modified: string
    usergroup: Usergroup
}

export interface Usergroup {
    id: number
    name: string
    description: string
    created: string
    modified: string
}

/****************************
 *    Generic interfaces    *
 ****************************/

export interface UserLocaleOption {
    i18n: string
    name: string
}

export interface UserDateformatsRoot {
    dateformats: SelectKeyValueString[]
    defaultDateFormat: string,
    timezones: UserTimezonesSelect[],
    serverTime: string,
    serverTimeZone: string
}

export interface LoadUsersByContainerIdRoot {
    users: SelectKeyValue[]
}

export interface UserTimezoneGroup {
    // Africa / America / Antarctica / Asia / Atlantic / Australia / Europe / Indian / Pacific ...
    [key: string]: UserTimezone
}

export interface UserTimezone {
    // Africa/Abidjan: Abidjan
    [key: string]: string
}

export interface UserTimezonesSelect {
    group: string // Africa / America / Antarctica / Asia / Atlantic / Australia / Europe / Indian / Pacific ...
    value: string // Africa/Abidjan
    name: string // Abidjan
}

export interface UserType {
    title: string
    color: ('primary' | 'secondary' | 'success' | 'danger' | 'warning' | 'info' | 'light' | 'dark')
    class: string
}

/**********************
 *    Login action    *
 **********************/
export interface LoginGetRoot {
    _csrfToken: string
    images: {
        description: string
        particles: string
        images: LoginImage[]
    }
    disableAnimation: boolean,
    hasValidSslCertificate: boolean
    isLoggedIn: boolean
    systemname: string
    logoUrl: string
    disableSocialButtons: boolean,
    errorMessages?: any[]
    successMessages?: any[]
}

export interface LoginImage {
    image: string
    credit: string
}

export interface UsersIndexParams {
    angular: true,
    scroll: boolean,
    sort: string,
    page: number,
    direction: 'asc' | 'desc' | '', // asc or desc
    'filter[Users.username]': string,
    'filter[Users.firstname]': string,
    'filter[Users.lastname]': string,
    'filter[Users.usergroup_id][]': number[]
}


export function getDefaultUsersIndexParams(): UsersIndexParams {
    return {
        angular: true,
        scroll: true,
        sort: 'Users.username',
        page: 1,
        direction: 'asc',
        'filter[Users.username]': '',
        'filter[Users.firstname]': '',
        'filter[Users.lastname]': '',
        'filter[Users.usergroup_id][]': []
    }
}

export interface UserPost {
    id?: number
    usergroup_id: number
    username: string
    firstname: string
    lastname: string
    showstatsinmenu: number
    paginatorlength: number
    dateformat: string
    timezone: string
    password: string
    confirm_password: string
}

export interface UsersEditResponse {
    user: UserPost
}
